<?php
namespace PME\Controllers\Custo;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;

class CustoRealPeriodoController extends \Zeedhi\Framework\Controller\Report {

    protected $birtStrategy;
    protected $environment;
    protected $crudService;

    public function __construct(ReportService $reportService, $birtStrategy, $environment, $crudService) {
        $this->birtStrategy  = $birtStrategy;
        $this->environment   = $environment;
        $this->crudService   = $crudService;

        parent::__construct($reportService);
    }

    protected $reportMapping = array (
      'PME08100' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array (
              "NMFILCENTPME"  => "P_NMFILCENTPME",
              "CDFILCENTPME"  => "P_CDFILCENTPME",
              "CDFILIAL"      => "P_CDFILIAL",
              "DTDE"          => "P_DATAINIC",
              "DTATE"         => "P_DATAFINA",
              "NMTIPOCUSTO"   => "P_NMCUSTO",
              "DTPERINI"      => "P_DTPERIINIC",
              "DTPERFIN"      => "P_DTPERIFINA",
              "ESTOCAVEIS"    => "P_IDESTOCAVEIS",
              "NAOESTOCAVEIS" => "P_IDNAOESTOC",
              "CDTIPOCUSTO"   => "P_NRTIPOCUSTO",
              "TAMACNIVEL"    => "P_NRTAMACNIVEL",
              "TAMCODIGO"     => "P_NRTAMCODIGO",
              "PRODUCTMASK"   => "P_PRODUCTMASK",
              "AGRUPMASK"     => "P_AGRUPMASK",
              "NRORG"         => "P_NRORG"
            )
          ),
      'PME08101' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array (
              "NMFILCENTPME"  => "P_NMFILCENTPME",
              "CDFILCENTPME"  => "P_CDFILCENTPME",
              "CDFILIAL"      => "P_CDFILIAL",
              "DTDE"          => "P_DATAINIC",
              "DTATE"         => "P_DATAFINA",
              "NMTIPOCUSTO"   => "P_NMCUSTO",
              "DTPERINI"      => "P_DTPERIINIC",
              "DTPERFIN"      => "P_DTPERIFINA",
              "ESTOCAVEIS"    => "P_IDESTOCAVEIS",
              "NAOESTOCAVEIS" => "P_IDNAOESTOC",
              "CDTIPOCUSTO"   => "P_NRTIPOCUSTO",
              "TAMACNIVEL"    => "P_NRTAMACNIVEL",
              "TAMCODIGO"     => "P_NRTAMCODIGO",
              "PRODUCTMASK"   => "P_PRODUCTMASK",
              "AGRUPMASK"     => "P_AGRUPMASK",
              "NRORG"         => "P_NRORG"
            )
            ),
        'PME08102' => 
            array (
                'strategy' => 'BIRT',
                'parameterMapping' => 
                array (
                  "NMFILCENTPME"  => "P_NMFILCENTPME",
                  "CDFILCENTPME"  => "P_CDFILCENTPME",
                  "CDFILIAL"      => "P_CDFILIAL",
                  "DTDE"          => "P_DATAINIC",
                  "DTATE"         => "P_DATAFINA",
                  "NMTIPOCUSTO"   => "P_NMCUSTO",
                  "DTPERINI"      => "P_DTPERIINIC",
                  "DTPERFIN"      => "P_DTPERIFINA",
                  "ESTOCAVEIS"    => "P_IDESTOCAVEIS",
                  "NAOESTOCAVEIS" => "P_IDNAOESTOC",
                  "CDTIPOCUSTO"   => "P_NRTIPOCUSTO",
                  "TAMACNIVEL"    => "P_NRTAMACNIVEL",
                  "TAMCODIGO"     => "P_NRTAMCODIGO",
                  "PRODUCTMASK"   => "P_PRODUCTMASK",
                  "AGRUPMASK"     => "P_AGRUPMASK",
                  "NRORG"         => "P_NRORG"
                )
               )

    );
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        $row['DTPERINI'] = $row['DTDE'];
        $row['DTPERFIN'] = $row['DTATE'];
        $row['NRORG'] = $this->environment->getNrOrgTrab();

        if(empty($row['UNIDADE'])) $row['CDFILIAL'] = $this->getAgregadasOper($row);
        else $row['CDFILIAL'] = join("_", $row['UNIDADE']);
        
        if($row['ORDENA'] == 0) $row['TAMCODIGO'] = 10;
        else $row['TAMCODIGO'] = 0;
        
        if($row['CD_REPORT_CODE'] == 'PME08102') $row['TAMACNIVEL'] = 0;

        $this->birtStrategy->setReportLocale($this->environment->getLanguage());
        $this->birtStrategy->setReportFormat(strtolower($row['reportFormat']));
    }

    private function getAgregadasOper($row){
        $conditions = [
          array(
            "columnName" => 'P_CDFILCENTPME',
            "operator"   => '=',
            "value"      => $row['CDFILCENTPME']
          ),
          array(
            "columnName" => 'P_CDOPERADOR',
            "operator"   => '=',
            "value"      => empty($row['NUTRICIONISTA']) ? null : join('_', $row['NUTRICIONISTA'])
          )
        ];

        $dataSet = $this->crudService->getFilteredDataSet("agregadasPorCentralOper", $conditions);  
        $rows = $dataSet->getRows();
        $unidades = '';
        foreach($rows as $unid){
            $unidades .= $unid['CDFILIAL'].'_';
        }
        return $unidades;
      }
}