import {
  Modal, ModalService, Grid, DialogService, Form, Select, Date as ZdDate, moment,
} from '@zeedhi/common';
import {
  Metadata, IDictionary, IEventParam, I18n,
  Loader, Http,
} from '@zeedhi/core';

export class Pme00609ImportacaoTpPrato {
  public utilities = Loader.getInstance('Utilities');
  
  private elaboracaoCardapio = Loader.getInstance('Pme00600ElaboracaoCardapio');
  
  public modalImportacao!: Modal;
  
  public selectedLine: IDictionary = {};
  
  // Funçoes search modal Importacao
  public async createPeriodoModal() {
    this.modalImportacao = await ModalService.createFromJson('/modals/gridPeriodoImportacao', true);
  }
  
  public openModalImportacao() {
    this.modalImportacao.show();
  }
  
  public destroyPeriodoModal() {
    this.modalImportacao.destroy();
  }
  
  public closeModal() {
    this.modalImportacao.hide();
  }

  // Funções da importação por tipo de prato.
  
  public openModalSearch() {
    const tipoCard = Metadata.getInstance<Select>('TIPOCARD').value;
    if (tipoCard) {
      this.openModalImportacao();
      this.utilities.setCdTipoCard(tipoCard);
      this.utilities.setFilterModalPeriodo();
    } else {
      this.utilities.showAlert('alertError', 'pme::tipocardNaoSelecionado', 'warning');
    }
  }
  
  public setPeriodoParams({ row }: IDictionary) {
    const periodoDe = Metadata.getInstance<ZdDate>('PERIODODE');
    const periodoAte = Metadata.getInstance<ZdDate>('PERIODOATE');
    const dtDe = Metadata.getInstance<ZdDate>('DTDE');
    const dtAte = Metadata.getInstance<ZdDate>('DTATE');
    periodoDe.value = row.DTINICARD;
    periodoAte.value = row.DTFINCARD;
    dtDe.value = row.DTINICARD;
    dtAte.value = row.DTFINCARD;
    this.modalImportacao.hide();
  }
  
  public clearFields({ component }: IEventParam<Select>) {
    const cardapio = Metadata.getInstance<Select>('TIPOCARD');
    const servico = Metadata.getInstance<Select>('SERVICO');
    const periodoDe = Metadata.getInstance<ZdDate>('PERIODODE');
    const periodoAte = Metadata.getInstance<ZdDate>('PERIODOATE');
    const dtDe = Metadata.getInstance<ZdDate>('DTDE');
    const dtAte = Metadata.getInstance<ZdDate>('DTATE');
    const incluirCard = Metadata.getInstance<ZdDate>('INCLUIRCARD');

    if (component.name === 'CENTRAL') {
      cardapio.setValue('');
      servico.setValue('');
      periodoDe.value = '';
      periodoAte.value = '';
      dtDe.value = '';
      dtAte.value = '';
      incluirCard.value = '';
    } else if (component.name === 'TIPOCARD') {
      servico.setValue('');
      periodoDe.value = '';
      periodoAte.value = '';
      dtDe.value = '';
      dtAte.value = '';
      incluirCard.value = '';
    }
  }
  
  public setFilterTpCardapio({ component }: IEventParam<Select>) {
    const central = Metadata.getInstance<Select>('CENTRAL');
    const filter: IDictionary = [];
    this.utilities.onMountedFilterFind(filter, 'CDFILCENTPME', '=', central.value);
    this.utilities.onMountedFilterFind(filter, 'NRPASTATCARD', '=', null);
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public setFilterServ({ component }: IEventParam<Select>) {
    const central = Metadata.getInstance<Select>('CENTRAL');
    const tipoCard = Metadata.getInstance<Select>('TIPOCARD');
    const filter = [
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: tipoCard.value ? tipoCard.value : '',
      },
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: central.value,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public validateDate(form: Form) {
    let valid = false;
    const row: IDictionary = form.value;
    const dtDe = row.DTDE;
    const dtAte = row.DTATE;
    const periodoDe = row.PERIODODE;
    const periodoAte = row.PERIODOATE;
    const concatDate = moment(periodoDe, 'DD/MM/YYYY').format(this.utilities.getFormatLocale()) + String(' a ') + moment(periodoAte, 'DD/MM/YYYY').format(this.utilities.getFormatLocale());
    
    if (this.utilities.dateIsValid(periodoDe, periodoAte) && this.utilities.dateIsValid(dtDe, dtAte)) {
      if (this.utilities.dateIsValid(periodoDe, dtDe) && this.utilities.dateIsValid(dtAte, periodoAte)) valid = true;
      else this.utilities.showAlert('alertError', I18n.translate('pme::dtCompreendidaPerCardImportacao', { PARAM1: concatDate }), 'error');
    } else this.utilities.showAlert('alertError', 'pme::dataIniMaior', 'error');
    return valid;
  }
  
  public validateDateIncluirCard(form: Form) {
    let valid = false;
    const dtCardIniPeriodo = this.elaboracaoCardapio.getFilterData.initialDate;
    const dtCardFinPeriodo = this.elaboracaoCardapio.getFilterData.finalDate;
    const concatDate = moment(dtCardIniPeriodo, 'DD/MM/YYYY').format(this.utilities.getFormatLocale()) + String(' a ') + moment(dtCardFinPeriodo, 'DD/MM/YYYY').format(this.utilities.getFormatLocale());
    const row: IDictionary = form.value;
    const incluirCard = row.INCLUIRCARD;
    
    if (incluirCard < dtCardIniPeriodo && incluirCard > dtCardFinPeriodo) this.utilities.showAlert('alertError', I18n.translate('pme::dtCompreendidaPerCardImportacao', { PARAM1: concatDate }), 'error');
    else valid = true;
    return valid;
  }
  
  public setFilterGridImportacao() {
    const gridImportacaoTpPrato = Metadata.getInstance<Grid>('gridImportacaoTpPrato');
    const form = Metadata.getInstance<Form>('formFilterImportTpPrato');
    const central = Metadata.getInstance<Select>('CENTRAL');
    const tipoCard = Metadata.getInstance<Select>('TIPOCARD');
    const servico = Metadata.getInstance<Select>('SERVICO');
    const periodoDe = Metadata.getInstance<ZdDate>('PERIODODE');
    const periodoAte = Metadata.getInstance<ZdDate>('PERIODOATE');
    const row = form.value;
    
    if (servico.value) {
      if (form.validate()) {
        if (this.validateDate(form)) {
          if (this.validateDateIncluirCard(form)) {
            Http.post('configCard/cardapioImportacao', { requestType: 'Row', row }).then((response) => {
              const result = response.data.dataset.result;
              if (result.length !== 0) {
                const filter = [
                  {
                    name: 'P_DTFINCARDPME',
                    operator: '=',
                    value: periodoAte.value,
                  },
                  {
                    name: 'P_NRCARDPME',
                    operator: '=',
                    value: result[0].NRCARDPME,
                  },
                  {
                    name: 'P_CDFILCENTPME',
                    operator: '=',
                    value: central.value,
                  },
                  {
                    name: 'P_CDSERVFILI',
                    operator: '=',
                    value: servico.value,
                  },
                  {
                    name: 'P_DTINICARDPME',
                    operator: '=',
                    value: periodoDe.value,
                  },
                  {
                    name: 'P_CDTIPOCARD',
                    operator: '=',
                    value: tipoCard.value,
                  },
                ];
                
                gridImportacaoTpPrato.datasource.filter = filter;
                gridImportacaoTpPrato.datasource.get();
              }
            });
          }
        } else gridImportacaoTpPrato.datasource.data = [];
      } else gridImportacaoTpPrato.datasource.data = [];
    }
  }
  
  public openDialog() {
    const grid = Metadata.getInstance<Grid>('gridImportacaoTpPrato');
    this.selectedLine = grid.selectedRows;
    if (this.selectedLine.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else {
      DialogService.show({
        name: 'pme_warningImportacao',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::acrescimoTpPrato',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.saveImportTpPrato(this.selectedLine) },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }
  
  public saveImportTpPrato(checkedRows: IDictionary) {
    const form = Metadata.getInstance<Form>('formFilterImportTpPrato');
    const rowForm = form.value;
    
    checkedRows.forEach((item: IDictionary) => {
      item.CENTRALDESTINO = this.elaboracaoCardapio.getFilterData.cdfilcentpme;
      item.SERVICODESTINO = this.elaboracaoCardapio.getFilterData.service.CDSERVFILI;
      item.TIPOCARDDESTINO = this.elaboracaoCardapio.getFilterData.typeMenu.CDTIPOCARD;
      item.NRCARDPMEDESTINO = this.elaboracaoCardapio.getFilterData.NRCARDPME;
      item.CDTIPOCARD = rowForm.TIPOCARD;
      item.DTDE = rowForm.DTDE;
      item.DTATE = rowForm.DTATE;
      item.INCLUIRCARD = rowForm.INCLUIRCARD;
    });
    
    Http.post('/configCard/saveImportTpPrato', { checkedRows }).then(() => {
      this.utilities.showAlert('alertSuccess', 'pme::salvoComSucesso', 'green');
      DialogService.hide();
    });
  }
}
