import {
  Config, Metadata, Loader, Http, IEventParam, IDictionary,
} from '@zeedhi/core';
import {
  Select, GridEditable, Form,
} from '@zeedhi/common';
import { PmeMask } from '@/config';

export class Pme11000PercentualAdesaoPratoController {
  private utilities = Loader.getInstance('Utilities');
  
  public appController = Loader.getInstance('AppController');
  
  public gridEditableController = Loader.getInstance('GridEditableController');
  
  public filterTipoCardapio({ component }: IEventParam<Select>) {
    const cardapio = Metadata.getInstance<Select>('TPCARD');
    const cdfilcentpme = this.appController.getCdCentral();
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: component.value,
      },
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: cdfilcentpme,
      },
      {
        name: 'NRPASTATCARD',
        operator: '=',
        value: 'T',
      },
    ];
    cardapio.datasource.filter = conditions;
    cardapio.datasource.get();
    this.reloadPercAct();
    // cardapio.datasource.get().then(() => {
    //   const firstRow = (cardapio.datasource.data.length > 0) ? cardapio.datasource.data[0] : {};
    //   cardapio.datasource.currentRow = firstRow;
    //   // this.reloadPercAct();
    // });
  }
  
  private reloadPercAct() {
    const cardapio = Metadata.getInstance<Select>('TPCARD');
    if (cardapio.datasource.data.length > 0) {
      this.filterPercAceit(); //  cardapio.datasource.data[0] como argumento
    }
  }
  
  public changeFields({ component }: IEventParam<Select>) {
    this.resetaPratos(component.value, Metadata.getInstance<Select>('PRATO_INICIAL'));
    this.resetaPratos(component.value, Metadata.getInstance<Select>('PRATO_FINAL'));
    this.filterTipoCardapio({ component });
  }
  
  private resetaPratos(cdfilial: string, prato: Select) {
    prato.datasource.filter = {
      CDFILIAL: cdfilial,
    };
    prato.datasource.get();
    if (prato.value !== null) prato.setValue(null);
  }
  
  public filterAgregada({ component }: IEventParam<Select>) {
    component.datasource.filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
    ];
    component.datasource.get();
  }
  
  public verifyPratIni({ component }: IEventParam<Select>) {
    const form = Metadata.getInstance<Form>('formFilter');
    if (component.value !== null) {
      if (this.utilities.returnPrat(form.getChildInstance<Select>('PRATO_INICIAL').value, component.value)) {
        this.utilities.showAlert('Alert', 'pme::pratoIniMenor', 'error');
        component.setValue(null);
      }
      this.reloadPercAct();
    }
  }
  
  public controllPratoIni() {
    const pratoFinal = Metadata.getInstance<Select>('PRATO_FINAL');
    if (pratoFinal.value !== null) pratoFinal.setValue(null);
    
    this.reloadPercAct();
  }
  
  public filterPercAceit() {
    const percPrato: GridEditable = Metadata.getInstance('PERACEITPRATO');
    const pratoIncial = Metadata.getInstance<Select>('PRATO_INICIAL').value;
    const pratoFinal = Metadata.getInstance<Select>('PRATO_FINAL').value;
    const tipoCard = Metadata.getInstance<Select>('TPCARD').selectValue;
    const nivelMaxPlate = (Config.masks.plateMask as PmeMask).nivelMax;
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: tipoCard.CDFILIAL,
      },
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDPRATOINI',
        operator: '=',
        value: pratoIncial !== null ? pratoIncial : '0',
      },
      {
        name: 'CDPRATOFIN',
        operator: '=',
        value: pratoFinal !== null ? `${pratoFinal}Z` : 'Z',
      },
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: tipoCard.CDTIPOCARD,
      },
      {
        name: 'MAXCDNVPRATO',
        operator: '=',
        value: nivelMaxPlate,
      },
    ];
    if (tipoCard.CDTIPOCARD) {
      percPrato.datasource.filter = conditions;
      percPrato.datasource.get();
      // percPrato.datasource.get().then(() => {
      //   const firstRow = (percPrato.datasource.data.length > 0) ? percPrato.datasource.data[0] : {};
      //   percPrato.datasource.currentRow = firstRow;
      // });
    }
  }
  
  public savePercAceita() {
    const grid: GridEditable = Metadata.getInstance('PERACEITPRATO');
    const editedRows = grid.getEditedRows();
    this.checkCentralNull(editedRows);
    
    if (editedRows.filter((data) => data.VRPERACEIT == null).length === 0 && editedRows.length > 0) {
      Http.post('/percAceita/save', { requestType: 'Row', editedRows }).then((response) => {
        if (response.data.dataset.save[0]) {
          grid.cancelEditedRows();
          Loader.getInstance('GridEditableController').onInlineEdit({ component: grid });
          grid.datasource.get();
          
          this.utilities.showAlert('Alert', 'pme::salvoComSucesso', 'success');
        }
      });
    } else {
      this.utilities.showAlert('Alert', 'pme::dadosNulos', 'error');
    }
  }
  
  private checkCentralNull(editedRows: IDictionary) {
    const centralNull = editedRows.filter((data: IDictionary) => data.CDFILCENTPME === null);
    
    if (centralNull.length > 0) {
      centralNull.forEach((row: IDictionary) => {
        row.CDFILCENTPME = this.appController.getCdCentral();
        row.NRORG = localStorage.getItem('NRORG');
        row.CDFILIAL = Metadata.getInstance<Select>('UNIDADE_AGREGADA').value;
        row.__is_new = true;
      });
    }
  }
  
  public cancelEditingTelaPercentual({ component }: IDictionary) {
    this.gridEditableController.cancelEditing({ component });
  }
}
