import {
  Loader, Metadata, IEventParam, IDictionary,
} from '@zeedhi/core';
import {
  Form, Select, Button,
} from '@zeedhi/common';

export class Pme01300EmissaoCardPerPAEController {
  public reportController = Loader.getInstance('ReportController');

  public appController = Loader.getInstance('AppController');

  public utilities = Loader.getInstance('Utilities');
  
  public modalController = Loader.getInstance('ModalController');

  public setTipoPrato({ component }: IEventParam<Select>) {
    const servico = Metadata.getInstance<Select>('SERVICO');
    const dadosCard = this.utilities.getParamsCard();
    const filter = [
      {
        name: 'P_NRCARDPME',
        operator: '=',
        value: dadosCard ? dadosCard.NRCARDPME : '',
      },
      {
        name: 'P_CDSERVFILI',
        operator: '=',
        value: servico.value.length > 0 ? servico.value.join('_') : null,
      },
      {
        name: 'P_CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public setFilterServ({ component }: IEventParam<Select>) {
    const tipoCard = Metadata.getInstance<Select>('TIPOCARD');
    const filter = [
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: tipoCard.value.CDTIPOCARD ? tipoCard.value.CDTIPOCARD : '',
      },
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }

  public clearTpPrato() {
    const tpPrato = Metadata.getInstance<Select>('TIPOPRAT');
    tpPrato.setValue('');
  }

  public generateReport({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('FORM');

    if (this.formIsValid(form)) {
      const row: IDictionary = form.value;
      
      this.setParams(row);
      if (!row.CDFILCENTPME) this.utilities.showAlert('alertError', 'pme::centralNaoInformada', 'error');
      else {
        row.reportFormat = component.name;
        this.utilities.activeLoadingButton(component);
        this.reportController.generateReport('report/emissaoCardPeriodoPAE', row, row.CD_REPORT_CODE).finally(() => {
          this.utilities.disableLoadingButton(component);
        });
      }
    }
  }
  
  private setParams(row: IDictionary) {
    /*
    A cada quatro tipos de pratos selecionados de um mesmo servico, deve gerar
    uma posicao na lista de servico. Exemplo: selecionado servico 10001 e os
    tipos de prato 01_02_03_04_05, neste caso a lista de servico recebe
    10001_10001 e a lista de tipo de prato 01_02_03_04|05
    */
    const paramsCard = this.utilities.getParamsCard();
    row.CDTIPOCARD = row.TIPOCARD.CDTIPOCARD;
    row.NMTIPOCARD = row.TIPOCARD.NMTIPOCARD;
    row.NRCARDPME = paramsCard.NRCARDPME;
    row.DSCARDPME = this.utilities.descCard;
    row.NMFILIAGRE = row.UNIDADE ? row.UNIDADE.NMFILIAL : null;
    row.CDFILIAL = row.UNIDADE ? row.UNIDADE.CDFILIAL : null;
    row.CDSERVFILI = row.SERVICO ? row.SERVICO : null;
    row.CDFILCENTPME = this.appController.getCdCentral();
    row.NMFILCENTPME = this.appController.getNmCentral();
    row.REPORT_NAME = row.REPORT.REPORT_NAME;
    row.CD_REPORT_CODE = row.REPORT.CD_REPORT_CODE;
  }

  private formIsValid(form: Form) {
    let valid = false;
    const row: IDictionary = form.value;
    const periodoDe = row.PERIODODE;
    const periodoAte = row.PERIODOATE;
    form.validate();
    if (periodoDe === '' || periodoAte === '') this.utilities.showAlert('alertError', 'pme::periodoObrig', 'error');
    else if (form.validate()) {
      if (this.utilities.dateIsValid(periodoDe, periodoAte)) valid = true;
      else this.utilities.showAlert('alertError', 'pme::dtCompreendidaPerCard', 'error');
    }
    return valid;
  }

  public openModalSearch() {
    const tipoCard = Metadata.getInstance<Select>('TIPOCARD');
    if (tipoCard.value) {
      this.modalController.openModal();
      this.utilities.setCdTipoCard(tipoCard.value.CDTIPOCARD);
    } else {
      this.utilities.showAlert('alertError', 'pme::tipocardNaoSelecionado', 'warning');
    }
  }

  public async createPeriodoModal() {
    this.modalController.createModal('modals/gridPeriodo');
  }

  public destroyPeriodoModal() {
    this.modalController.destroyModal();
  }
}
